@@ -228,6 +228,8 @@ static void phonebook_size_result(const char *buffer, size_t bufsize,
 	struct aparam_header *hdr = (struct aparam_header *) aparam;
 	uint16_t phonebooksize;
 
+	DBG("vcards %d", vcards);
+
 	phonebooksize = htons(vcards);
 
 	hdr->tag = PHONEBOOKSIZE_TAG;
@@ -244,6 +246,8 @@ static void query_result(const char *buffer, size_t bufsize, int vcards,
 {
 	struct pbap_session *pbap = user_data;
 
+	DBG("");
+
 	if (!pbap->buffer)
 		pbap->buffer = g_string_new_len(buffer, bufsize);
 	else
@@ -282,21 +286,6 @@ static int alpha_sort(gconstpointer a, gconstpointer b)
 	return g_strcmp0(e1->name, e2->name);
 }
 
-static int phonetical_sort(gconstpointer a, gconstpointer b)
-{
-	const struct cache_entry *e1 = a;
-	const struct cache_entry *e2 = b;
-
-	/*
-	 * SOUND attribute is optinal. Keep the order
-	 * when this attribute is not available.
-	 */
-	if (!e1->sound)
-		return 1;
-
-	return g_strcmp0(e1->sound, e2->sound);
-}
-
 static int indexed_sort(gconstpointer a, gconstpointer b)
 {
 	const struct cache_entry *e1 = a;
@@ -305,6 +294,18 @@ static int indexed_sort(gconstpointer a, gconstpointer b)
 	return (e1->handle - e2->handle);
 }
 
+static int phonetical_sort(gconstpointer a, gconstpointer b)
+{
+	const struct cache_entry *e1 = a;
+	const struct cache_entry *e2 = b;
+
+	/* SOUND attribute is optional. Use Indexed sort if not present. */
+	if (!e1->sound || !e2->sound)
+		return indexed_sort(a, b);
+
+	return g_strcmp0(e1->sound, e2->sound);
+}
+
 static GSList *sort_entries(GSList *l, uint8_t order, uint8_t search_attrib,
 							const char *value)
 {
@@ -373,6 +374,8 @@ static void cache_ready_notify(void *user_data)
 	GSList *l;
 	uint16_t max = pbap->params->maxlistcount;
 
+	DBG("");
+
 	if (max == 0) {
 		/* Ignore all other parameter and return PhoneBookSize */
 		char aparam[4];
@@ -422,9 +425,16 @@ static void cache_entry_done(void *user_data)
 	const char *id;
 	int ret;
 
+	DBG("");
+
 	pbap->cache.valid = TRUE;
 
 	id = cache_find(&pbap->cache, pbap->find_handle);
+	if (id == NULL) {
+		debug("Entry %d not found on cache", pbap->find_handle);
+		obex_object_set_io_flags(pbap, G_IO_ERR, -ENOENT);
+		return;
+	}
 
 	ret = phonebook_get_entry(pbap->folder, id, pbap->params,
 						query_result, pbap);
@@ -502,6 +512,11 @@ static struct apparam_field *parse_aparam(const uint8_t *buffer, uint32_t hlen)
 		len += hdr->len + sizeof(struct aparam_header);
 	}
 
+	DBG("o %x sa %x sv %s fil %" G_GINT64_MODIFIER "x for %x max %x off %x",
+			param->order, param->searchattrib, param->searchval,
+			param->filter, param->format, param->maxlistcount,
+			param->liststartoffset);
+
 	return param;
 
 failed:
@@ -538,6 +553,8 @@ static int pbap_get(struct obex_session *os, obex_object_t *obj,
 	ssize_t rsize;
 	int ret;
 
+	DBG("name %s type %s pbap %p", name, type, pbap);
+
 	if (type == NULL)
 		return -EBADR;
 
@@ -601,6 +618,9 @@ static int pbap_setpath(struct obex_session *os, obex_object_t *obj,
 
 	name = obex_get_name(os);
 
+	DBG("name %s folder %s nonhdr 0x%x%x", name, pbap->folder,
+							nonhdr[0], nonhdr[1]);
+
 	fullname = phonebook_set_folder(pbap->folder, name, nonhdr[0], &err);
 	if (err < 0)
 		return err;
@@ -661,6 +681,9 @@ static void *vobject_pull_open(const char *name, int oflag, mode_t mode,
 	phonebook_cb cb;
 	int ret;
 
+	DBG("name %s context %p maxlistcount %d", name, context,
+						pbap->params->maxlistcount);
+
 	if (oflag != O_RDONLY) {
 		ret = -EPERM;
 		goto fail;
@@ -693,6 +716,8 @@ static void *vobject_list_open(const char *name, int oflag, mode_t mode,
 	struct pbap_session *pbap = context;
 	int ret;
 
+	DBG("name %s context %p valid %d", name, context, pbap->cache.valid);
+
 	if (oflag != O_RDONLY) {
 		ret = -EPERM;
 		goto fail;
@@ -732,6 +757,8 @@ static void *vobject_vcard_open(const char *name, int oflag, mode_t mode,
 	uint32_t handle;
 	int ret;
 
+	DBG("name %s context %p valid %d", name, context, pbap->cache.valid);
+
 	if (oflag != O_RDONLY) {
 		ret = -EPERM;
 		goto fail;
@@ -779,6 +806,9 @@ static ssize_t vobject_pull_read(void *object, void *buf, size_t count,
 {
 	struct pbap_session *pbap = object;
 
+	DBG("buffer %p maxlistcount %d", pbap->buffer,
+						pbap->params->maxlistcount);
+
 	if (!pbap->buffer)
 		return -EAGAIN;
 
@@ -797,6 +827,9 @@ static ssize_t vobject_list_read(void *object, void *buf, size_t count,
 {
 	struct pbap_session *pbap = object;
 
+	DBG("valid %d maxlistcount %d", pbap->cache.valid,
+						pbap->params->maxlistcount);
+
 	/* Backend still busy reading contacts */
 	if (!pbap->cache.valid)
 		return -EAGAIN;
@@ -814,6 +847,8 @@ static ssize_t vobject_vcard_read(void *object, void *buf, size_t count,
 {
 	struct pbap_session *pbap = object;
 
+	DBG("buffer %p", pbap->buffer);
+
 	if (!pbap->buffer)
 		return -EAGAIN;
 
